<?php

require_once 'phpgen_settings.php';
require_once 'components/application.php';
require_once 'components/security/permission_set.php';
require_once 'components/security/user_authentication/table_based_user_authentication.php';
require_once 'components/security/grant_manager/user_grant_manager.php';
require_once 'components/security/grant_manager/composite_grant_manager.php';
require_once 'components/security/grant_manager/hard_coded_user_grant_manager.php';
require_once 'components/security/grant_manager/table_based_user_grant_manager.php';
require_once 'components/security/table_based_user_manager.php';

include_once 'components/security/user_identity_storage/user_identity_session_storage.php';

require_once 'database_engine/pgsql_engine.php';

$grants = array();

$appGrants = array();

$dataSourceRecordPermissions = array();

$tableCaptions = array('qHome' => 'Version',
'public.contact' => 'Kunden',
'public.contact.public.contact_relationship' => 'Kunden->Referenz',
'public.contact.public.card' => 'Kunden->Vertrag',
'public.contact.public.activity' => 'Kunden->Bewegungen',
'public.contact.public.invoice' => 'Kunden->Rechnungen',
'public.contact.public.invoice.public.invoice_item' => 'Kunden->Rechnungen->Rechnungszeilen',
'public.activity' => 'Bewegungen',
'qWMS_checkout' => 'Bewegungsabschluß',
'public.tariff_edm_config' => 'Periodenvorlage',
'public.contact_edm_gln' => 'GLN Vorlagen',
'public.activity_edm_stock' => 'Lagerbestände',
'public.activity_edm' => 'Mengen Rohdaten',
'public.activity_edm_result' => 'Logistik Meldung Basis',
'public.qedm_result' => 'Logistik Meldung Detail',
'public.invoice' => 'Rechnungen',
'public.invoice.public.invoice_item' => 'Rechnungen->Rechnungszeilen',
'public.invoice.public.notes' => 'Rechnungen->Notizen',
'public.cash' => 'Kassabuchungen',
'qWMS_activity_all' => 'alle Buchungen',
'qWMS_activity_contact_tarif_day' => 'Buchungen Kunde/Tarif/Tag',
'qWMS_activity_contact_tarif_month' => 'Buchungen Kunde/Tarif/Monat',
'qWMS_activity_contact_tarif_year' => 'Buchungen Kunde/Tarif/Jahr',
'qWMS_activity_tarif_day' => 'Buchungen Tarif/Tag',
'qWMS_activity_tarif_month' => 'Buchungen Tarif/Monat',
'qWMS_activity_tarif_year' => 'Buchungen Tarif/Jahr',
'qReports' => 'Berichte',
'qServices' => 'Dienste',
'qelfinder' => 'Dateimanager',
'public.tariff_edm_waste_catalog' => 'Abfallkatalog',
'public.tariff_edm_activity_types' => 'Abfallbuchungsarten',
'public.salutation' => 'Anreden',
'public.user_app_perms' => 'WMO Berechtigungen',
'public.report' => 'Berichte',
'public.unit' => 'Einheiten',
'public.preset' => 'Einstellungen',
'public.finance' => 'Finanzkonten',
'public.category' => 'Kategorien',
'public.appliance' => 'Infrastruktur',
'public.appliance.public.appliance_config' => 'Infrastruktur->Konfiguration',
'public.deliverytype' => 'Lieferarten',
'public.pricetable' => 'Preistabelle',
'public.pricetable.public.pricetable_entry' => 'Preistabelle->Preise',
'public.invoicetype' => 'Rechnungsarten',
'public.tariff' => 'Tarife',
'public.tariff.public.tariff_finance' => 'Tarife->Kunde/Konto',
'public.title' => 'Titel',
'public.currency' => 'Währungen',
'public.paymenttype' => 'Zahlungsarten');

$usersTableInfo = array(
    'TableName' => 'public.users',
    'UserId' => 'user_id',
    'UserName' => 'user_name',
    'Password' => 'user_password',
    'Email' => '',
    'UserToken' => '',
    'UserStatus' => ''
);

function EncryptPassword($password, &$result)
{

}

function VerifyPassword($enteredPassword, $encryptedPassword, &$result)
{

}

function BeforeUserRegistration($username, $email, $password, &$allowRegistration, &$errorMessage)
{

}    

function AfterUserRegistration($username, $email)
{

}    

function PasswordResetRequest($username, $email)
{

}

function PasswordResetComplete($username, $email)
{

}

function CreatePasswordHasher()
{
    $hasher = CreateHasher('SHA256');
    if ($hasher instanceof CustomStringHasher) {
        $hasher->OnEncryptPassword->AddListener('EncryptPassword');
        $hasher->OnVerifyPassword->AddListener('VerifyPassword');
    }
    return $hasher;
}

function CreateTableBasedGrantManager()
{
    global $tableCaptions;
    global $usersTableInfo;
    $userPermsTableInfo = array('TableName' => 'public.user_perms', 'UserId' => 'user_id', 'PageName' => 'page_name', 'Grant' => 'perm_name');
    
    $tableBasedGrantManager = new TableBasedUserGrantManager(PgConnectionFactory::getInstance(), GetGlobalConnectionOptions(),
        $usersTableInfo, $userPermsTableInfo, $tableCaptions, false);
    return $tableBasedGrantManager;
}

function CreateTableBasedUserManager() {
    global $usersTableInfo;
    return new TableBasedUserManager(PgConnectionFactory::getInstance(), GetGlobalConnectionOptions(), $usersTableInfo, CreatePasswordHasher(), false);
}

function SetUpUserAuthorization()
{
    global $grants;
    global $appGrants;
    global $dataSourceRecordPermissions;

    $hasher = CreatePasswordHasher();

    $hardCodedGrantManager = new HardCodedUserGrantManager($grants, $appGrants);
    $tableBasedGrantManager = CreateTableBasedGrantManager();
    $grantManager = new CompositeGrantManager();
    $grantManager->AddGrantManager($hardCodedGrantManager);
    if (!is_null($tableBasedGrantManager)) {
        $grantManager->AddGrantManager($tableBasedGrantManager);
    }

    $userAuthentication = new TableBasedUserAuthentication(new UserIdentitySessionStorage(), false, $hasher, CreateTableBasedUserManager(), true, false, false);

    GetApplication()->SetUserAuthentication($userAuthentication);
    GetApplication()->SetUserGrantManager($grantManager);
    GetApplication()->SetDataSourceRecordPermissionRetrieveStrategy(new HardCodedDataSourceRecordPermissionRetrieveStrategy($dataSourceRecordPermissions));
}
